/*
This program uses the c compiler library functions to configure and
use the UART1 and ADC12 on the SBD.

In this program, the UART1 is configured for a 9600 baud rate
At a constant rate, the pot values connected to RP1,RP2 and RP3  
are displayed in hex via the serial port.

When SW1 is pressed the pot values start tranmission
When SW2 is pressed the transmission stops
*/

#define __dsPIC30F6014__
// #include<p30f6014>


#include <uart.h>
#include <adc12.h>
#include <p30F6014.h>

//---------------------------------------------------------------------------
// Configuration bits

   _FOSC(CSW_FSCM_OFF & XT_PLL4);   //XT with 4xPLL oscillator, Failsafe clock off
   _FWDT(WDT_OFF);                  //Watchdog timer disabled
   _FBORPOR(PBOR_OFF & MCLR_EN);    //Brown-out reset disabled, MCLR reset enabled
   _FGS(CODE_PROT_OFF);             //Code protect disabled

//---------------------------------------------------------------------------


#define BAUDRATE 9600		// desired baud rate
#define FCY  7372800			// xtal = 7.3728Mhz; PLLx4
#define SW1	!PORTAbits.RA12
#define SW2	!PORTAbits.RA13
#define LF	0x0A
#define CR	0x0D
#define NULL 0x00
#define SPACE 0x20
#define WRITE_CHAR	0xA8
#define HOME_CLEAR	0x82
#define CURSOR_ON		0x8C

unsigned char RxValue;

// interrupt vector for UART1 TX interrupt
void __attribute__((__interrupt__)) _U1TXInterrupt(void)
{
	IFS0bits.U1TXIF = 0;
}

// interrupt vector for UART1 RX interrupt
void __attribute__((__interrupt__)) _U1RXInterrupt(void)
{
	IFS0bits.U1RXIF = 0;	// clear interrupt flag
	RxValue = (char)U1RXREG;	// read input register
	LCD_Display_Byte(WRITE_CHAR);	// display on LCD
	LCD_Display_Byte(RxValue);		// --do--
}


void InitADC12(void);
void LoadADC(unsigned int offset);
void InitUART1(void);
void AverageADC(void);
void InitTMR3(void);


unsigned int RPValue[] = {1,2,3};


char ADCdata[] = {'R','P','1',SPACE,'=',SPACE,'1','2','3',CR,LF,
						'R','P','2',SPACE,'=',SPACE,'1','2','3',CR,LF,
						'R','P','3',SPACE,'=',SPACE,'1','2','3',CR,LF,						
						SPACE,CR,LF,NULL};

int main(void)
{


unsigned char TxIndex;
unsigned char i;

	InitTMR3();	
	InitADC12();
	LCD_Display_Setup();	// Init LCD display
	LCD_Display_Byte(HOME_CLEAR);	// clear LCD and put cursor at home
	LCD_Display_Byte(CURSOR_ON); // turn on cursor
	T3CONbits.TON = 1; //turn on TMR3
	InitUART1();
	while(1)			// do for ever
		{
		TxIndex = 0;		// point to first char in the ADC buffer
/*******************************************************************/
		while (!IFS0bits.ADIF); 	// conversion done?
		IFS0bits.ADIF = 0;			// clear flag
 		/** once the conversion is completed, get the data **/
		AverageADC(); 						//get average ADC value from buffer
		for (i = 0; i<=2; i++)
			LoadADC(i);						// Load  USART buffer
		while (ADCdata[TxIndex])	// and transmit
			{
			WriteUART1((int)ADCdata[TxIndex++]);
			while(BusyUART1());
			}
		}	// end of while (1)

}

/*******************************************************************
	Below is the code required to setup the TMR3 to match every 10 mS
********************************************************************/
void InitTMR3(void)
{
	T3CON = 0x0010;			//Tcy/8 internal clock
	TMR3 = 0;
	PR3 = 0x2400;				// time out for 10 mS
}
	
/*******************************************************************
		Below is the code required to setup the ADC registers for :
		1. 3 channel scan AN6, AN5 and AN4
		2. Auto Sample using TMR3 set for 10 mS sample
		3. Interrupt after 12 load of samples
		4. Conversion Tad = 1uS = 15 uS
		5. Manual check of interrupt flag.

																		
*********************************************************************/
void InitADC12(void)
{

 ADPCFG = 0xFF8F;				// all PORTB = Digital; RB6-RB4 = analog
 ADCON1 = 0x0044;				// Auto convert using TMR3 sample for 10 mS ...
									// then convert
 ADCON2 = 0x042C;				// scan inputs and interrupt after 12 samples
 ADCSSL = 0x0070;				// Scan inputs: AN6, AN5 and AN4
 ADCON3 = 0x0000F;			// TMR3 = 10ms , Tad = 8Tcy = 1uS
 
 
 ADCON1bits.ADON = 1;			// turn ADC ON
}


/********************************************************************

	AverageADC() averages the 4 values saved in the buffer in RP2,RP3
	and RP1
*********************************************************************/
void AverageADC(void)
{

	RPValue[0] = ADCBUF2 + ADCBUF5 + ADCBUF8 + ADCBUFB;
	RPValue[0] = RPValue[0] >> 2;
	RPValue[1] = ADCBUF0 + ADCBUF3 + ADCBUF6 + ADCBUF9;
	RPValue[1] = RPValue[1] >> 2;
	RPValue[2] = ADCBUF1 + ADCBUF4 + ADCBUF7 + ADCBUFA;
	RPValue[2] = RPValue[2] >> 2;
}

/********************************************************************

	LoadADC() converts the 12-bit value sampled on the RP1 pot
	to a 3 digit BCD value and loads it into the ADCdata buffer 

*********************************************************************/
void LoadADC(unsigned int offset)
{
unsigned char ADCbcd;
unsigned int j;

	j = offset;
	offset = offset*11 + 6;
	while(BusyUART1());						// wait till transmit is over
	ADCbcd = (char)(RPValue[j] >> 8);		// convert the ADC value 
	if (ADCbcd > 9)
		ADCbcd = ADCbcd + 0x37;
	else ADCbcd = ADCbcd + 0x30;
	ADCdata[offset] = ADCbcd;
	ADCbcd = (char)(RPValue[j] >> 4);
	ADCbcd = ADCbcd & 0x0F;
	if (ADCbcd > 9)
		ADCbcd = ADCbcd + 0x37;
	else ADCbcd = ADCbcd + 0x30;
	ADCdata[offset+1] = ADCbcd;
	ADCbcd = (char)RPValue[j] & 0x0F;
	if (ADCbcd > 9)
		ADCbcd = ADCbcd + 0x37;
	else ADCbcd = ADCbcd + 0x30;
	ADCdata[offset+2] = ADCbcd;

}

void InitUART1(void)
{

unsigned int baudvalue;
unsigned int U1MODEvalue;
unsigned int U1STAvalue;

	CloseUART1();
	ConfigIntUART1(UART_RX_INT_EN & UART_RX_INT_PR6 & 
						UART_TX_INT_DIS & UART_TX_INT_PR2);
	U1MODEvalue = 	UART_EN & UART_IDLE_CON &
						UART_DIS_WAKE & UART_EN_LOOPBACK &
						UART_EN_ABAUD & UART_NO_PAR_8BIT &
						UART_1STOPBIT;
	U1STAvalue =	UART_INT_TX_BUF_EMPTY &
						UART_TX_PIN_NORMAL &
						UART_TX_ENABLE & UART_INT_RX_CHAR &
						UART_ADR_DETECT_DIS &
						UART_RX_OVERRUN_CLEAR;
	baudvalue = ((FCY/16)/BAUDRATE) - 1;
	OpenUART1(U1MODEvalue, U1STAvalue, baudvalue);
}


